#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "utils.h"
#include "f_xy.h"
#include "dsi.h"
#include <sys\timeb.h> 
#include "sha1.h"

unsigned char tadsrl_keyX[16] = {0x4a, 0x00, 0x00, 0x4e, 0x4e, 0x00, 0x00, 0x4a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
unsigned char tadsrl_keyY[16] = {0xcc, 0xfc, 0xa7, 0x03, 0x20, 0x61, 0xbe, 0x84, 0xd3, 0xeb, 0xa4, 0x26, 0xb8, 0x6d, 0xbe, 0xc2};
unsigned char sd_key[16] = {0x3d, 0xa3, 0xea, 0x33, 0x4c, 0x86, 0xa6, 0xb0, 0x2a, 0xae, 0xdb, 0x51, 0x16, 0xea, 0x92, 0x62};
char modcrypt_shared_key[8] = {'N','i','n','t','e','n','d','o'};
// middle two words are 'NINTENDO' big endian
u32 emmc_keyX_3DS[4] = {0x00000000, 0x544E494E, 0x4F444E45, 0x00000000};
u32 emmc_keyY[4] = {0x0AB9DC76,0xBD4DC4D3,0x202DDD1D,0xE1A00005};
unsigned char block[0x10];
int i;

void decrypt_modcrypt_area(dsi_context* ctx, unsigned char *buffer, unsigned int size)
{
	uint32_t len = size / 0x10;
	while(len>0)
	{
		memset(block, 0, 0x10);
		dsi_crypt_ctr_block(ctx, buffer, block);
		memcpy(buffer, block, 0x10);
		buffer+=0x10;
		len--;
	}
}

// From dsi_srl_extract
int decryptsrl(unsigned char *srl)
{
	unsigned char *keyX_ptr = NULL, *keyY_ptr = NULL;
	uint32_t offset, size;
	int verbose=0;
	unsigned char *header, *buffer;
	unsigned char key_x[16];
	unsigned char key_y[16];
	unsigned char key[16];
	dsi_context ctx;

	header = srl;

	memcpy(key_x, modcrypt_shared_key, 8);

	memcpy(&key_x[8], &header[0x0c], 4);
	key_x[12 + 0] = header[0x0c + 3];
	key_x[12 + 1] = header[0x0c + 2];
	key_x[12 + 2] = header[0x0c + 1];
	key_x[12 + 3] = header[0x0c + 0];
	memcpy(key_y, &header[0x350], 16);
	
	if((header[0x1c] & 4) || (header[0x1bf] & 0x80))
	{
		printf("Crypting dev modcrypt.\n");
	}
	else
	{
		printf("Crypting retail modcrypt.\n");
		keyX_ptr = key_x;
		keyY_ptr = key_y;
	}
	memcpy(key, header, 16);

	printf("Crypting...\n");
	if(keyX_ptr)
	{
		F_XY((uint32_t*)key, (uint32_t*)key_x, (uint32_t*)key_y);
	}
	dsi_set_key(&ctx, key);

	
	memcpy(&offset, &header[0x220], 4);
	memcpy(&size, &header[0x224], 4);
	dsi_set_ctr(&ctx, &header[0x300]);

	if(offset!=0)
	{
		printf("Modcrypt area 0: offset %x size %x\n", offset, size);
		buffer = srl + offset;
		decrypt_modcrypt_area(&ctx, buffer, size);
	}
	else
	{
		printf("Modcrypt area 0 is unused.\n");
	}


	memcpy(&offset, &header[0x228], 4);
	memcpy(&size, &header[0x22c], 4);
	dsi_set_ctr(&ctx, &header[0x314]);

	if(offset!=0)
	{
		printf("Modcrypt area 1: offset %x size %x\n", offset, size);
		buffer = srl + offset;
		decrypt_modcrypt_area(&ctx, buffer, size);
	}
	else
	{
		printf("Modcrypt area 1 is unused.\n");
	}

	printf("Done.\n");
	return 0;
}

void decrypt_boot2_section(u8* data, u32 len, u8 is3DS)
{
    unsigned char normalkey[16] = {0};
    unsigned char keyX_TWLFIRM[16] = {0xE1, 0xEB, 0xDF, 0x44, 0xAB, 0x1D, 0x81, 0xE3, 0x93, 0x9A, 0x4A, 0xB5, 0x36, 0xFC, 0x3A, 0x0E};
    unsigned char keyX_DSi[16] = {0x4E, 0x69, 0x6E, 0x74, 0x65, 0x6E, 0x64, 0x6F, 0x20, 0x44, 0x53, 0x00, 0x01, 0x23, 0x21, 0x00};
    unsigned char keyY_3DS[16] = {0xAB, 0x4E, 0x18, 0xA8, 0x51, 0x16, 0x90, 0x7E, 0x9F, 0x65, 0xF0, 0xCE, 0x21, 0x7C, 0x3A, 0x70};
    unsigned char keyY_DSi[16] = {0xEC, 0x07, 0x00, 0x00, 0x34, 0xE2, 0x94, 0x7C, 0xC3, 0x0E, 0x81, 0x7C, 0xEC, 0x07, 0x00, 0x00};
    u32 ctr[4];
    memset(ctr, 0, 16);

    int i;

    dsi_context dsictx;
    if(is3DS)
        F_XY((u32*) normalkey, (u32*)keyX_TWLFIRM, (u32*)keyY_3DS);
    else
        F_XY((u32*) normalkey, (u32*)keyX_DSi, (u32*)keyY_DSi);

    ctr[0] = len;
    ctr[1] = -len;
    ctr[2] = ~len;
    printf("CTR:\n");
    hexdump(ctr,16);

    printf("Normalkey:\n");
    hexdump(normalkey,16);
    dsi_set_key(&dsictx, normalkey);
    dsi_set_ctr(&dsictx, (unsigned char*)ctr);

    // auto-increments ctr
    for(i = 0; i < len; i+= 0x10)
       dsi_crypt_ctr_block(&dsictx, data+i, data+i);
}

void decrypt_boot2(char* in, u8 is3DS)
{
    u32 offset;
    u32 len;
    u8* data;
    FILE* f_in = fopen(in,"r+b");
    FILE* f_out;
    if(f_in == NULL)
    {
        printf("Input filename invalid!");
        return;
    }
    
    // decrypt and write ARM9
    f_out = fopen("arm9.bin","wb");
    fseek(f_in, 0x220, SEEK_SET);
    fread(&offset, 1, sizeof(offset), f_in);
    fseek(f_in, 0x22C, SEEK_SET);
    fread(&len, 1, sizeof(len), f_in);
    
    fseek(f_in, offset, SEEK_SET);
    data = malloc(len);
    fread(data, 1, len, f_in);
    decrypt_boot2_section(data, len, is3DS);
    fwrite(data, 1, len, f_out);
    free(data);
    fclose(f_out);
    
    // decrypt and write ARM7
    f_out = fopen("arm7.bin","wb");
    fseek(f_in, 0x230, SEEK_SET);
    fread(&offset, 1, sizeof(offset), f_in);
    fseek(f_in, 0x23C, SEEK_SET);
    fread(&len, 1, sizeof(len), f_in);
    
    fseek(f_in, offset, SEEK_SET);
    data = malloc(len);
    fread(data, 1, len, f_in);
    decrypt_boot2_section(data, len, is3DS);
    fwrite(data, 1, len, f_out);
    fclose(f_out);
    fclose(f_in);
    free(data);
    
}

void decrypt_srl(char* in, char* out)
{
    FILE* f_in = fopen(in,"r+b");
    FILE* f_out;
    if(!strcmp(in,out))
        f_out = fopen(out,"r+b");
    else
        f_out = fopen(out,"wb");
    if(f_in == NULL)
    {
        printf("Input filename invalid!");
        return;
    }
    
    fseek(f_in, 0L, SEEK_END);
    int sz = ftell(f_in);
    fseek(f_in, 0L, SEEK_SET);
    unsigned char* srl = malloc(sz);
    fread(srl, 1, sz, f_in);
    
    decryptsrl(srl);
    
    fwrite(srl, 1, sz, f_out);
    free(srl);
    fclose(f_in);
    fclose(f_out);
}

void cid_brute_3ds(u32* consoleID, u8* emmc_cid, u8* test_data, u8 isN3DS)
{
    dsi_context ctx;
    int i, diff;
    struct timeb start, end;
    u8 emmc_normalkey[16];
    u8 emmc_cid_hash[20];
    u8 CTR[16];
    
    // TODO: untested, might be 0x02000000
    if(isN3DS)
        consoleID[1] = 0x00000002;
    else
        consoleID[1] = 0x00000000;
    

    u8 target_bytes[16] = {0};
    sha1(emmc_cid_hash, emmc_cid, 16);
    memcpy(CTR, emmc_cid_hash, 16);
    
    ftime(&start);
    // first bit is always set, just brute the bottom 7
    for(i = 0x00000000; i < 0x7FFFFFFF; i++) {
        consoleID[0] = i;
        
        dsi_set_ctr(&ctx, (u8*)CTR);
        dsi_add_ctr(&ctx, 0x1E);
        emmc_keyX_3DS[0] = (consoleID[0] ^ 0xB358A6AF) | 0x80000000;
        emmc_keyX_3DS[3] = consoleID[1] ^ 0x08C267B7;
        F_XY((u32*) emmc_normalkey, (u32*) emmc_keyX_3DS, (u32*) emmc_keyY);
        dsi_set_key(&ctx, emmc_normalkey);
        
        dsi_crypt_ctr_block(&ctx, test_data, block);
        
        if(!memcmp(target_bytes, block, sizeof(target_bytes))){
            printf("Got it!! %08X%08X", consoleID[0], consoleID[1]);
            break;
        }
        
        if(!(i % 0x100000))
            printf("CID 0x%08X of 0x7FFFFFFF\n",i);
    }
    ftime(&end);
    diff = (int) (1000.0 * (end.time - start.time)
        + (end.millitm - start.millitm));
    printf("\nOperation took %u milliseconds\n", diff);
}

void nand_decrypt_3ds(u8 *emmc_cid, u32 *consoleID, char *in, char *out, u8 brute_cid, u8 isN3DS)
{
    dsi_context ctx;
    u32 i;
    u8 emmc_normalkey[16];
    u8 emmc_cid_hash[20];
    u8 CTR[16];
    u8 brute_buf[16];
    
    if(brute_cid)
    {
        FILE* f_in = fopen(in,"r+b");
        fseek(f_in, 0x1E0, SEEK_SET);
        fread(brute_buf, 1, sizeof(brute_buf), f_in);
        fclose(f_in);
        cid_brute_3ds(consoleID, emmc_cid, brute_buf, isN3DS);
    }
    
    // Prepare CTR by SHA1-hashing eMMC CID
    sha1(emmc_cid_hash, emmc_cid, 16);
    memcpy(CTR, emmc_cid_hash, 16);
    dsi_set_ctr(&ctx, (u8*)CTR);

    // Generate AES normalkey from consoleID
    emmc_keyX_3DS[0] = (consoleID[0] ^ 0xB358A6AF) | 0x80000000;
    emmc_keyX_3DS[3] = consoleID[1] ^ 0x08C267B7;
    F_XY((u32*) emmc_normalkey, (u32*) emmc_keyX_3DS, (u32*) emmc_keyY);
    dsi_set_key(&ctx, emmc_normalkey);
    printf("Normalkey: \n");
    hexdump(emmc_normalkey,16);
    
    FILE* f_in = fopen(in,"r+b");
    FILE* f_out;
    if(!strcmp(in,out))
        f_out = fopen(out,"r+b");
    else
        f_out = fopen(out,"wb");
    if(f_in == NULL)
    {
        printf("Input filename invalid!");
        return;
    }
    for(i = 0; i < 0x0B100000; i += 0x10) {
        fread(block, 1, 0x10, f_in);
        dsi_crypt_ctr_block(&ctx, block, block);
        fwrite(block, 1, 0x10, f_out);
        if(i % 0x1000000 == 0)
            printf("%.2f %% complete.\n",(100.0 * i / 0x0B100000));
    }
    fclose(f_in);
    fclose(f_out);
    printf("Decrypt done!");
}

void nand_decrypt_dsi(u8 *emmc_cid, u32 *consoleID, char *in, char *out)
{
    dsi_context ctx;
    u32 i;
    u32 emmc_keyX[4];
    u8 emmc_normalkey[16];
    u8 emmc_cid_hash[20];
    u8 CTR[16];
    
    // Prepare AES CTR by SHA1-hashing eMMC CID
    sha1(emmc_cid_hash, emmc_cid, 16);
    memcpy(CTR, emmc_cid_hash, 16);
    dsi_set_ctr(&ctx, (u8*)CTR);
    
    
    // Endian-swap the input ConsoleID
    consoleID[0] = getbe32((u8*)consoleID);
    consoleID[1] = getbe32((u8*)consoleID+4);    
    
    // Generate AES normalkey from consoleID (which comes in reverse word order)
    emmc_keyX[0] = consoleID[1];
    emmc_keyX[1] = consoleID[1] ^ 0x24EE6906;
    emmc_keyX[2] = consoleID[0] ^ 0xE65B601D;
    emmc_keyX[3] = consoleID[0];
    F_XY((u32*) emmc_normalkey, (u32*) emmc_keyX, (u32*) emmc_keyY);
    dsi_set_key(&ctx, emmc_normalkey);
    printf("Normalkey: \n");
    hexdump(emmc_normalkey,16);
    
    FILE* f_in = fopen(in,"r+b");
    FILE* f_out;
    if(!strcmp(in,out))
        f_out = fopen(out,"r+b");
    else
        f_out = fopen(out,"wb");
    if(f_in == NULL)
    {
        printf("Input filename invalid!");
        return;
    }
    for(i = 0; i < 0x0F000000; i += 0x10) {
        fread(block, 1, 0x10, f_in);
        dsi_crypt_ctr_block(&ctx, block, block);
        fwrite(block, 1, 0x10, f_out);
        if(i % 0x1000000 == 0)
            printf("%.2f %% complete.\n",(100.0 * i / 0x0F000000));
    }
    fclose(f_in);
    fclose(f_out);
    printf("Decrypt done!");
}

void display_help()
{
    printf("Usage: twltool <command> [args]\n");
    printf("Commands:\n");
    printf("  nandcrypt\n");
    printf("  modcrypt\n");
    printf("  boot2\n");
    printf("nandcrypt: (de)crypt DSi NAND\n");
    printf("  --cid [hex cid]               eMMC CID\n");
    printf("  --consoleid [hex consoleid]   DSi ConsoleID\n");
    printf("  --in [infile]                 Input image\n");
    printf("  --out [outfile]               Output file (optional)\n");
    printf("  --3ds                         Crypt 3DS TWLNAND\n");
    printf("    --3dsbrute                  Bruteforce 3DS ConsoleID\n");
    printf("      --n3ds                    New3DS bruteforce\n");
    printf("modcrypt: (de)crypt SRL modcrypt sections\n");
    printf("  --in [infile]                 Input SRL\n");
    printf("  --out [outfile]               Output file (optional)\n");
    printf("boot2: decrypt boot2 image to arm7.bin and arm9.bin\n");
    printf("  --in [infile]                 Input image\n");
    printf("  --3ds                         Crypt 3DS boot2\n");
}

int main(int argc, char* argv[])
{
    unsigned char consoleID[8] = {0};
    unsigned char cid[16] = {0};
    char in[400] = {0};
    char out[400] = {0};
    char is3DS = 0;
    char brute_cid = 0;
    char isN3DS = 0;
    
    printf("TWLTool v1.1\n");
    printf("  by WulfyStylez\n");
    printf("  Special thanks to CaitSith2\n\n");
    if(argc <= 1)
        display_help();
    else
    {
        if(!strcmp(argv[1], "nandcrypt"))
        {
            if(argc < 6) {
                printf("Invalid options!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            for(i = 0; i < argc; i++)
            {
                if(!strcmp(argv[i],"--consoleid")) {
                    if(hex2bytes(argv[i+1], strlen(argv[i+1]), consoleID, 8))
                        exit(EXIT_FAILURE);
                }
                if(!strcmp(argv[i],"--cid")) {
                    if(hex2bytes(argv[i+1], strlen(argv[i+1]), cid, 16))
                        exit(EXIT_FAILURE);
                }
                if(!strcmp(argv[i],"--3ds"))
                    is3DS = 1;
                if(!strcmp(argv[i],"--3dsbrute"))
                    brute_cid = 1;
                if(!strcmp(argv[i],"--in")) {
                    strcpy(in, argv[i+1]);
                }
                if(!strcmp(argv[i],"--out")) {
                    strcpy(out, argv[i+1]);
                }
            }
            if(in[0] == 0) {
                printf("Invalid filename!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            if(out[0] == 0)
                strcpy(out,in);
            if(is3DS)
                nand_decrypt_3ds(cid, (u32*)consoleID, in, out, brute_cid, isN3DS);
            else
                nand_decrypt_dsi(cid, (u32*)consoleID, in, out);
        }
        else if(!strcmp(argv[1], "modcrypt"))
        {
            if(argc < 4) {
                printf("Invalid options!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            for(i = 0; i < argc; i++)
            {
                if(!strcmp(argv[i],"--in")) {
                    strcpy(in, argv[i+1]);
                }
                if(!strcmp(argv[i],"--out")) {
                    strcpy(out, argv[i+1]);
                }
            }
            if(in[0] == 0) {
                printf("Invalid filename!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            if(out[0] == 0)
                strcpy(out,in);
            decrypt_srl(in, out);
        }
        else if(!strcmp(argv[1], "boot2"))
        {
            if(argc < 4) {
                printf("Invalid options!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            for(i = 0; i < argc; i++)
            {
                if(!strcmp(argv[i],"--in")) {
                    strcpy(in, argv[i+1]);
                }
                if(!strcmp(argv[i],"--3ds")) {
                    is3DS = 1;
                }
            }
            if(in[0] == 0) {
                printf("Invalid filename!\n");
                display_help();
                exit(EXIT_FAILURE);
            }
            decrypt_boot2(in, is3DS);
        }
        else
        {
            printf("Invalid command!\n");
            display_help();
            exit(EXIT_FAILURE);
        }
    }
    
	return 0;
}
